"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .definitionfield import DefinitionField, DefinitionFieldTypedDict
from .integrationclient import IntegrationClient
from .method import Method
from .mode import Mode
from .paymentserviceconfiguration import (
    PaymentServiceConfiguration,
    PaymentServiceConfigurationTypedDict,
)
from .requiredcheckoutfields import (
    RequiredCheckoutFields,
    RequiredCheckoutFieldsTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentServiceDefinitionTypedDict(TypedDict):
    id: str
    r"""The definition ID of the payment service that can be configured. This is the underlying provider followed by a dash followed by the method."""
    display_name: str
    r"""A human friendly name for this service."""
    method: Method
    fields: List[DefinitionFieldTypedDict]
    r"""A list of credentials and related fields which can be configured for this service."""
    reporting_fields: List[DefinitionFieldTypedDict]
    r"""A list of reporting fields which can be configured for this service."""
    supported_currencies: List[str]
    r"""A list of three-letter ISO currency codes that this service supports."""
    supported_countries: List[str]
    r"""A list of two-letter ISO country codes that this service supports."""
    mode: Mode
    supported_features: Dict[str, bool]
    r"""Features supported by the payment service."""
    required_checkout_fields: List[RequiredCheckoutFieldsTypedDict]
    r"""A list of condition that define when some fields must be provided with a transaction request."""
    configuration: PaymentServiceConfigurationTypedDict
    supported_integration_clients: Nullable[List[IntegrationClient]]
    r"""List of supported integration clients. Defaults to redirect for most redirect connectors."""
    type: Literal["payment-service-definition"]
    r"""Always `payment-service-definition`."""
    icon_url: NotRequired[Nullable[str]]
    r"""An icon to display for the payment service."""


class PaymentServiceDefinition(BaseModel):
    id: str
    r"""The definition ID of the payment service that can be configured. This is the underlying provider followed by a dash followed by the method."""

    display_name: str
    r"""A human friendly name for this service."""

    method: Method

    fields: List[DefinitionField]
    r"""A list of credentials and related fields which can be configured for this service."""

    reporting_fields: List[DefinitionField]
    r"""A list of reporting fields which can be configured for this service."""

    supported_currencies: List[str]
    r"""A list of three-letter ISO currency codes that this service supports."""

    supported_countries: List[str]
    r"""A list of two-letter ISO country codes that this service supports."""

    mode: Mode

    supported_features: Dict[str, bool]
    r"""Features supported by the payment service."""

    required_checkout_fields: List[RequiredCheckoutFields]
    r"""A list of condition that define when some fields must be provided with a transaction request."""

    configuration: PaymentServiceConfiguration

    supported_integration_clients: Nullable[List[IntegrationClient]]
    r"""List of supported integration clients. Defaults to redirect for most redirect connectors."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["payment-service-definition"]],
            AfterValidator(validate_const("payment-service-definition")),
        ],
        pydantic.Field(alias="type"),
    ] = "payment-service-definition"
    r"""Always `payment-service-definition`."""

    icon_url: OptionalNullable[str] = UNSET
    r"""An icon to display for the payment service."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "icon_url"]
        nullable_fields = ["icon_url", "supported_integration_clients"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class VoidableFieldTypedDict(TypedDict):
    key: str
    value: str


class VoidableField(BaseModel):
    key: str

    value: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from gr4vy.utils import (
    FieldMetadata,
    HeaderMetadata,
    QueryParamMetadata,
    validate_const,
)
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class ListBuyerPaymentMethodsGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListBuyerPaymentMethodsGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""


OrderBy = Union[
    Literal[
        "asc",
        "desc",
    ],
    UnrecognizedStr,
]
r"""The direction to sort the payment methods in."""


class ListBuyerPaymentMethodsRequestTypedDict(TypedDict):
    buyer_id: NotRequired[Nullable[str]]
    r"""The ID of the buyer to query payment methods for."""
    buyer_external_identifier: NotRequired[Nullable[str]]
    r"""The external identifier of the buyer to query payment methods for."""
    sort_by: Nullable[Literal["last_used_at"]]
    r"""The field to sort the payment methods by."""
    order_by: NotRequired[OrderBy]
    r"""The direction to sort the payment methods in."""
    country: NotRequired[Nullable[str]]
    r"""The country code to filter payment methods by. This only applies to payment methods with a `country` value."""
    currency: NotRequired[Nullable[str]]
    r"""The currency code to filter payment methods by. This only applies to payment methods with a `currency` value."""
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListBuyerPaymentMethodsRequest(BaseModel):
    buyer_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The ID of the buyer to query payment methods for."""

    buyer_external_identifier: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The external identifier of the buyer to query payment methods for."""

    SORT_BY: Annotated[
        Annotated[
            OptionalNullable[Literal["last_used_at"]],
            AfterValidator(validate_const("last_used_at")),
        ],
        pydantic.Field(alias="sort_by"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "last_used_at"
    r"""The field to sort the payment methods by."""

    order_by: Annotated[
        Optional[OrderBy],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "desc"
    r"""The direction to sort the payment methods in."""

    country: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The country code to filter payment methods by. This only applies to payment methods with a `country` value."""

    currency: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The currency code to filter payment methods by. This only applies to payment methods with a `currency` value."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "buyer_id",
            "buyer_external_identifier",
            "sort_by",
            "order_by",
            "country",
            "currency",
            "merchant_account_id",
        ]
        nullable_fields = [
            "buyer_id",
            "buyer_external_identifier",
            "sort_by",
            "country",
            "currency",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .reportspecmodel import ReportSpecModel
from gr4vy.types import BaseModel
from typing import Any, Dict
from typing_extensions import TypedDict


class ReportSpecTypedDict(TypedDict):
    model: ReportSpecModel
    params: Dict[str, Any]
    r"""The parameters for the report model."""


class ReportSpec(BaseModel):
    model: ReportSpecModel

    params: Dict[str, Any]
    r"""The parameters for the report model."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class GiftCardRequestTypedDict(TypedDict):
    r"""The number and pin for a gift card to fetch a balance for."""

    number: str
    r"""The 16-19 digit number for the gift card."""
    pin: str
    r"""The PIN for this gift card."""


class GiftCardRequest(BaseModel):
    r"""The number and pin for a gift card to fetch a balance for."""

    number: str
    r"""The 16-19 digit number for the gift card."""

    pin: str
    r"""The PIN for this gift card."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cartitem import CartItem, CartItemTypedDict
from .paymentlinkstatus import PaymentLinkStatus
from .shippingdetails import ShippingDetails, ShippingDetailsTypedDict
from .statementdescriptor import StatementDescriptor, StatementDescriptorTypedDict
from .transactionbuyer import TransactionBuyer, TransactionBuyerTypedDict
from .transactionintent import TransactionIntent
from .transactionpaymentsource import TransactionPaymentSource
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentLinkTypedDict(TypedDict):
    id: str
    r"""The unique identifier for the payment link."""
    url: str
    r"""The URL for the payment link."""
    amount: int
    r"""The amount for the payment link."""
    country: str
    r"""The country code for the payment link."""
    currency: str
    r"""The currency code for the payment link."""
    intent: TransactionIntent
    cart_items: Nullable[List[CartItemTypedDict]]
    r"""The cart items for the payment link."""
    payment_source: TransactionPaymentSource
    r"""The way payment method information made it to this transaction."""
    created_at: datetime
    r"""The date and time the payment link was created."""
    updated_at: datetime
    r"""The date and time the payment link was last updated."""
    status: PaymentLinkStatus
    type: Literal["payment-link"]
    r"""Always `payment-link`."""
    expires_at: NotRequired[Nullable[datetime]]
    r"""The expiration date and time for the payment link."""
    external_identifier: NotRequired[Nullable[str]]
    r"""The merchant reference for the payment link."""
    statement_descriptor: NotRequired[Nullable[StatementDescriptorTypedDict]]
    r"""The statement descriptor for the payment link."""
    locale: NotRequired[Nullable[str]]
    r"""The locale for the payment link."""
    merchant_name: NotRequired[Nullable[str]]
    r"""The merchant's display name."""
    merchant_url: NotRequired[Nullable[str]]
    r"""The merchant's website URL."""
    merchant_banner_url: NotRequired[Nullable[str]]
    r"""The merchant's banner image URL."""
    merchant_color: NotRequired[Nullable[str]]
    r"""The merchant's brand color."""
    merchant_message: NotRequired[Nullable[str]]
    r"""A message from the merchant."""
    merchant_terms_and_conditions_url: NotRequired[Nullable[str]]
    r"""URL to the merchant's terms and conditions."""
    merchant_favicon_url: NotRequired[Nullable[str]]
    r"""URL to the merchant's favicon."""
    return_url: NotRequired[Nullable[str]]
    r"""The return URL after payment completion."""
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    r"""Arbitrary metadata for the payment link."""
    buyer: NotRequired[Nullable[TransactionBuyerTypedDict]]
    r"""The buyer associated with the payment link."""
    shipping_details: NotRequired[Nullable[ShippingDetailsTypedDict]]
    r"""The shipping details for the payment link."""
    connection_options: NotRequired[Nullable[Dict[str, Dict[str, Any]]]]
    r"""The connection options for the payment link."""
    store: NotRequired[bool]
    r"""Whether the payment method was stored."""
    buyer_id: NotRequired[Nullable[str]]
    r"""The ID of the buyer to associate with the stored payment method."""


class PaymentLink(BaseModel):
    id: str
    r"""The unique identifier for the payment link."""

    url: str
    r"""The URL for the payment link."""

    amount: int
    r"""The amount for the payment link."""

    country: str
    r"""The country code for the payment link."""

    currency: str
    r"""The currency code for the payment link."""

    intent: TransactionIntent

    cart_items: Nullable[List[CartItem]]
    r"""The cart items for the payment link."""

    payment_source: TransactionPaymentSource
    r"""The way payment method information made it to this transaction."""

    created_at: datetime
    r"""The date and time the payment link was created."""

    updated_at: datetime
    r"""The date and time the payment link was last updated."""

    status: PaymentLinkStatus

    TYPE: Annotated[
        Annotated[
            Optional[Literal["payment-link"]],
            AfterValidator(validate_const("payment-link")),
        ],
        pydantic.Field(alias="type"),
    ] = "payment-link"
    r"""Always `payment-link`."""

    expires_at: OptionalNullable[datetime] = UNSET
    r"""The expiration date and time for the payment link."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""The merchant reference for the payment link."""

    statement_descriptor: OptionalNullable[StatementDescriptor] = UNSET
    r"""The statement descriptor for the payment link."""

    locale: OptionalNullable[str] = UNSET
    r"""The locale for the payment link."""

    merchant_name: OptionalNullable[str] = UNSET
    r"""The merchant's display name."""

    merchant_url: OptionalNullable[str] = UNSET
    r"""The merchant's website URL."""

    merchant_banner_url: OptionalNullable[str] = UNSET
    r"""The merchant's banner image URL."""

    merchant_color: OptionalNullable[str] = UNSET
    r"""The merchant's brand color."""

    merchant_message: OptionalNullable[str] = UNSET
    r"""A message from the merchant."""

    merchant_terms_and_conditions_url: OptionalNullable[str] = UNSET
    r"""URL to the merchant's terms and conditions."""

    merchant_favicon_url: OptionalNullable[str] = UNSET
    r"""URL to the merchant's favicon."""

    return_url: OptionalNullable[str] = UNSET
    r"""The return URL after payment completion."""

    metadata: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Arbitrary metadata for the payment link."""

    buyer: OptionalNullable[TransactionBuyer] = UNSET
    r"""The buyer associated with the payment link."""

    shipping_details: OptionalNullable[ShippingDetails] = UNSET
    r"""The shipping details for the payment link."""

    connection_options: OptionalNullable[Dict[str, Dict[str, Any]]] = UNSET
    r"""The connection options for the payment link."""

    store: Optional[bool] = False
    r"""Whether the payment method was stored."""

    buyer_id: OptionalNullable[str] = UNSET
    r"""The ID of the buyer to associate with the stored payment method."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "expires_at",
            "external_identifier",
            "statement_descriptor",
            "locale",
            "merchant_name",
            "merchant_url",
            "merchant_banner_url",
            "merchant_color",
            "merchant_message",
            "merchant_terms_and_conditions_url",
            "merchant_favicon_url",
            "return_url",
            "metadata",
            "buyer",
            "shipping_details",
            "connection_options",
            "store",
            "buyer_id",
        ]
        nullable_fields = [
            "expires_at",
            "external_identifier",
            "statement_descriptor",
            "locale",
            "merchant_name",
            "merchant_url",
            "merchant_banner_url",
            "merchant_color",
            "merchant_message",
            "merchant_terms_and_conditions_url",
            "merchant_favicon_url",
            "return_url",
            "cart_items",
            "metadata",
            "buyer",
            "shipping_details",
            "connection_options",
            "buyer_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .giftcarderrorcode import GiftCardErrorCode
from .giftcardredemptionstatus import GiftCardRedemptionStatus
from .transactiongiftcard import TransactionGiftCard, TransactionGiftCardTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GiftCardRedemptionTypedDict(TypedDict):
    id: str
    r"""The ID for the gift card redemption."""
    status: GiftCardRedemptionStatus
    amount: int
    r"""The amount redeemed for this gift card."""
    refunded_amount: int
    r"""The amount refunded for this gift card. This can not be larger than `amount`."""
    gift_card: TransactionGiftCardTypedDict
    type: Literal["gift-card-redemption"]
    r"""Always `gift-card-redemption`."""
    gift_card_service_redemption_id: NotRequired[Nullable[str]]
    r"""The gift card service's unique ID for the redemption."""
    error_code: NotRequired[Nullable[GiftCardErrorCode]]
    r"""If this gift card redemption resulted in an error, this will contain the internal code for the error."""
    raw_error_code: NotRequired[Nullable[str]]
    r"""If this gift card redemption resulted in an error, this will contain the raw error code received from the gift card provider."""
    raw_error_message: NotRequired[Nullable[str]]
    r"""If this gift card redemption resulted in an error, this will contain the raw error message received from the gift card provider."""


class GiftCardRedemption(BaseModel):
    id: str
    r"""The ID for the gift card redemption."""

    status: GiftCardRedemptionStatus

    amount: int
    r"""The amount redeemed for this gift card."""

    refunded_amount: int
    r"""The amount refunded for this gift card. This can not be larger than `amount`."""

    gift_card: TransactionGiftCard

    TYPE: Annotated[
        Annotated[
            Optional[Literal["gift-card-redemption"]],
            AfterValidator(validate_const("gift-card-redemption")),
        ],
        pydantic.Field(alias="type"),
    ] = "gift-card-redemption"
    r"""Always `gift-card-redemption`."""

    gift_card_service_redemption_id: OptionalNullable[str] = UNSET
    r"""The gift card service's unique ID for the redemption."""

    error_code: OptionalNullable[GiftCardErrorCode] = UNSET
    r"""If this gift card redemption resulted in an error, this will contain the internal code for the error."""

    raw_error_code: OptionalNullable[str] = UNSET
    r"""If this gift card redemption resulted in an error, this will contain the raw error code received from the gift card provider."""

    raw_error_message: OptionalNullable[str] = UNSET
    r"""If this gift card redemption resulted in an error, this will contain the raw error message received from the gift card provider."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "gift_card_service_redemption_id",
            "error_code",
            "raw_error_code",
            "raw_error_message",
        ]
        nullable_fields = [
            "gift_card_service_redemption_id",
            "error_code",
            "raw_error_code",
            "raw_error_message",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class AccountUpdaterJobCreateTypedDict(TypedDict):
    payment_method_ids: List[str]
    r"""A list of payment method IDs to request an update for."""


class AccountUpdaterJobCreate(BaseModel):
    payment_method_ids: List[str]
    r"""A list of payment method IDs to request an update for."""
